/* exporter.c
 * export some memory via RSM & monitor */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <rsmapi.h>
#include "import_export.h"

#define DEBUG  1
#define CHARBUF  1024
#define CONTROLLER 0
#define TRUSTED_PERMISSIONS  0666

/* some global variables */



myMessage_t* createMessage(void)
  {
  myMessage_t       *theMessage ;

  /* allocate space equal to the size of the data structure plus
     the size of the message buffer */
  theMessage = (myMessage_t *)valloc(sizeof(myMessage_t) + CHARBUF);

  /*theMessage->content = (char *)valloc(CHARBUF);*/
  strcpy((char *)theMessage->content,"all is well");

  theMessage->number = 1;
  theMessage->length = strlen((char *)theMessage->content);

  return theMessage;
  }



const char* getCreateExportError(int status)
  {
  switch( status)
    {
    case(RSMERR_BAD_SEG_HNDL) : 
               return("Invalid segment handle.");
    case(RSMERR_BAD_CTLR_HNDL) : 
               return("Invalid controller handle.");
    case(RSMERR_CTLR_NOT_PRESENT) : 
               return("Controller not present.");
    case(RSMERR_BAD_LENGTH) : 
               return("Length zero or length exceeds controller limits.");
    case(RSMERR_BAD_ADDR) : 
               return("Invalid address.");
    case(RSMERR_INSUFFICIENT_MEM) : 
               return("Insufficient memory.");
    case(RSMERR_INSUFFICIENT_RESOURCES) : 
               return("Insufficient resources.");
    case(RSMERR_PERM_DENIED) : 
               return("Permission denied.");
    case(RSMERR_NOT_CREATOR) : 
               return("Not creator of segment.");
    case(RSMERR_REBIND_NOT_ALLOWED) : 
               return("Rebind not allowed.");
    case(RSMERR_BAD_MEM_ALIGNMENT) : 
               return("The address is not aligned on a page boundary.");
    default:
               return("unkown status");
    }
  }


rsm_memseg_export_handle_t*  createExport(rsm_topology_t  *rsmTP,
                                           volatile myMessage_t *theMessage)
  {
  rsm_memseg_export_handle_t  *theRSMsegment ;
  rsmapi_controller_handle_t  theController ;
  rsmapi_controller_attr_t    *ctlAttr ;
  size_t                      ctlPageSize, msgSize, segSize, sizeDiff;
  int   								status ;
  uint_t 							exportFlags;

  
  /* get controller */
  theController = getController(rsmTP,CONTROLLER );


  /* find controller page size, etc */

  fprintf(stderr,"getting attributes of controller %s\n",
                 rsmTP->connections[CONTROLLER]->hdr.cntlr_name);

  /* must allocate memory for controller attributes */
  ctlAttr = (rsmapi_controller_attr_t *)malloc(sizeof(rsmapi_controller_attr_t));
  if (ctlAttr == NULL)
    {
    fprintf(stderr,"no memory to allocate for controller attributes\n");
    exit(1);
    }

  status = rsm_get_controller_attr(theController, ctlAttr);
  if ( status )
    {
    fprintf(stderr,"error getting controller attributes (status %d): %s\n",
            status,getControllerError(status));
    exit(1);
    }

  
  /* nb: length on exported segment must be multiple of _controller_ page 
  /* size in bytes, so even up size of message to page size */
  ctlPageSize = ctlAttr->attr_page_size;
  fprintf(stderr,"rsm page is %d bytes\n",ctlPageSize);

  msgSize = sizeof theMessage ;
  fprintf(stderr,"message is %d bytes\n",msgSize);

  segSize = msgSize;

  if ( sizeDiff = (msgSize % ctlPageSize))
    {
    fprintf(stderr,"difference is %d bytes\n",sizeDiff);
    segSize += (ctlPageSize - sizeDiff) ; 
    }
  

  fprintf(stderr,"trying to create segment of %d bytes\n",segSize);


  /* createMemorySegment(); */

  exportFlags =  RSM_ALLOW_REBIND ;
  status = rsm_memseg_export_create(theController, theRSMsegment,
                                    (void *)theMessage, (size_t)segSize , 
                                    exportFlags);
  if (status)
    {
    fprintf(stderr,"error creating segment (status %d): %s\n",
            status,  getCreateExportError(status));
    exit(1);
    }
  fprintf(stderr,"segment made\n");


  return(theRSMsegment);
  }




rsm_memseg_id_t   publishSegment(rsm_topology_t  *rsmTP,
                                 rsm_memseg_export_handle_t *theRSMsegment)
  {
  int                         n, status, numRemoteCtl;
  rsmapi_access_entry_t			*theAccessList;
  rsm_memseg_id_t 				theSegmentID;


  /* set up access list */
  fprintf(stderr,"getting number of remote controllers\n");

  numRemoteCtl = rsmTP->connections[CONTROLLER]->hdr.remote_cntlr_count ;

  fprintf(stderr,"sizing theAccessList for %d remote controllers\n",
                  numRemoteCtl);

  theAccessList = (rsmapi_access_entry_t *)valloc(sizeof(rsmapi_access_entry_t)
                                                     * numRemoteCtl ) ;

  for ( n = 0 ; n <  numRemoteCtl ; n++)
    {
    theAccessList[n].ae_node = 
           rsmTP->connections[CONTROLLER]->remote_cntlr[n].remote_nodeid ;
    theAccessList[n].ae_permission = TRUSTED_PERMISSIONS ;
    }

  /* segment ID can be given as a pointed to a variable with value 0, and
     the system assigns a unique id, but there is no way to communicate
     this to the remote node(s), so we used a fixed segment id */

  theSegmentID = SEGMENT_ID ;
  status = rsm_memseg_export_publish(*theRSMsegment, &theSegmentID,
                                      theAccessList, (uint_t)numRemoteCtl );
  if (status)
    {
    perror("error exporting segment");
    exit(1);
    }
  
  return theSegmentID ;
  }





void main (int argc, char **argv)
  {
  /* "volatile" indicates that this is shared memory - the compiler
     mustn't take shortcuts */
  volatile  myMessage_t       *theMessage ;
  rsm_topology_t 					*rsmTP ;
  rsm_memseg_export_handle_t  *theRSMsegment ;
  rsm_memseg_id_t             theSegmentID = 0 ;
  int                         status;
  
  
  rsmTP = getTopology(); 

  theMessage = createMessage();

  theRSMsegment = createExport(rsmTP, theMessage);

  /* publish the segment */
  theSegmentID = publishSegment(rsmTP,theRSMsegment);
  

  /* checkSegment(); */

  while ( 1 )
    {
    sleep(3);
    printf("message %d is --> %s\n", theMessage->number, theMessage->content);
    }

  }
