/* common functions for export and import */

/* segment ID cannot be auto-discovered, so we have to sepcify one */
#define SEGMENT_ID  0x400000

typedef struct {
  int number ;
  int length ;
  char content[1] ;
  }  myMessage_t ;



rsm_topology_t* getTopology(void)
  {
  rsm_topology_t  *rsmTopology ;
  int status ;

  status = rsm_get_interconnect_topology(&rsmTopology);
  if ( status )
    {
    perror("error trying to get topology");
    exit(1);
    }
  return rsmTopology ;
  }



const char*  getControllerError(int status)
  {
  switch (status)
    {
    case (RSMERR_BAD_CTLR_HNDL) :
		return("Invalid controller handle.");
    case (RSMERR_BAD_ADDR) :
		return("Bad address");
    case (RSMERR_CTLR_NOT_PRESENT) :
		return("Controller not present.");
    case (RSMERR_INSUFFICIENT_MEM) :
		return("Insufficient memory.");
    case (RSMERR_BAD_LIBRARY_VERSION) :
		return("Invalid library version.");
    default:
               return("unknown status");
    }
  }



rsmapi_controller_handle_t  getController(rsm_topology_t *rsmTP, 
                                          int ctlrIndex)
  {
  rsmapi_controller_handle_t  theController;
  int				               status ;

  /* get controller */
  status = rsm_get_controller( rsmTP->connections[ctlrIndex]->hdr.cntlr_name,
                               &theController);
  if ( status )
    {
    fprintf(stderr,"error getting controller (status %d): %s\n",
            status,getControllerError(status));
    exit(1);
    }

  return(theController);
  }



typedef struct 
   {
   int 					count;
   rsm_node_id_t	   *nodes;
   } rsmNodeIDList_t;


/* return a list of nodes on a given connection */
rsmNodeIDList_t*  getNodes(rsm_topology_t *rsmTP, int ctlrIndex)
  {
  rsmNodeIDList_t					*theList;
  int									n;

  theList =  (rsmNodeIDList_t *)malloc(sizeof(rsmNodeIDList_t));
  if ( theList == NULL )
     {
     fprintf(stderr,"couldn't allocate memory for node id list\n");
     exit(1);
     }

  theList->count = (int)(rsmTP->connections[ctlrIndex]->hdr.remote_cntlr_count);

  theList->nodes = calloc((size_t)(theList->count), sizeof(rsm_node_id_t) );
                                              
  for ( n=0 ; n < theList->count ; n++)
    {
    theList->nodes[n] = 
           rsmTP->connections[ctlrIndex]->remote_cntlr[n].remote_nodeid ;
    }

  return theList ;
  }

