/* importer.c
   
   program that imports rsm segment as then does things to it */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <rsmapi.h>
#include <sys/types.h>
#include <time.h>

#include "import_export.h"

#define DEBUG  1
#define CHARBUF  1024
#define CONTROLLER 0
#define TRUSTED_PERMISSIONS  0666
#define READ_WRITE_PERM  0600

/* some global variables */



rsm_memseg_import_handle_t  connectSegment(rsm_topology_t *rsmTP)
  {
  rsmapi_controller_handle_t		theCtlr ;
  rsm_memseg_import_handle_t		importHandle;
  rsmNodeIDList_t						*nodeList ;
  rsm_memseg_id_t						theSegmentID ;
  int										status, n ;
  
  theSegmentID =  SEGMENT_ID ;

  /* get controller handle */
  theCtlr = getController( rsmTP, CONTROLLER ) ;

  /* find the other nodes on this section */
  nodeList = getNodes(rsmTP, CONTROLLER ) ;

  if ( ! nodeList->count )
    {
    fprintf(stderr,"no other nodes connected via %s",
                     rsmTP->connections[CONTROLLER]->hdr.cntlr_name );
    return(NULL);
    }
  

  /* try accessing nodes at the given segmentID until we find something */
  for ( n=0 ; n < nodeList->count ; n++)
    {
    status = rsm_memseg_import_connect(theCtlr, nodeList->nodes[n],
                                       theSegmentID, READ_WRITE_PERM,
                                       &importHandle);

    if (status == 0)
      {
      fprintf(stderr,"made connection to node id %d\n", nodeList->nodes[n]);
      break ;
      }
  
    switch (status)
      {
      case (RSMERR_SEG_NOT_PUBLISHED_TO_NODE): ;  /* fallthru */
      case (RSMERR_SEG_NOT_PUBLISHED): ;          /* fallthru */
      case (RSMERR_REMOTE_NODE_UNREACHABLE): 
                fprintf(stderr,"segment not available at node %d\n",
                                                         nodeList->nodes[n]);
                break;

      default:
           fprintf(stderr,"error importing segment (status %d)\n",status);
           exit(1);
      }
                                       
    }


  return importHandle ;
  }


void changeMessage(volatile myMessage_t *theMessage)
  {
  time_t clock;

  /* barrier start should go here */
  theMessage->number++;

  clock = time(0);
  strcpy((char *)theMessage->content,ctime(&clock));
  
  theMessage->length = strlen((char *)theMessage->content);
  /* barrier end/update should go here */
  
  
  }


void main(int argc, char **argv)
  {
  rsm_topology_t					*rsmTP ;
  int									status;
  rsm_memseg_import_handle_t	importHandle;
  volatile myMessage_t			*theMessage ;

  rsmTP = getTopology();


  /* connect to shared memory segment */

  importHandle = connectSegment(rsmTP);

  /* map shared segment to local memory */

  status = rsm_memseg_import_map(importHandle,(void **)&theMessage, 
                                RSM_MAP_NONE, READ_WRITE_PERM,
                                 0,8192);
  if ( status)
    {
    fprintf(stderr,"error mapping imported memory (status %d)\n",status);
    exit(1);
    }
  

  /* change values */

  while ( 1 )
    {
    printf("message %d is --> %s\n", theMessage->number,theMessage->content);
    sleep(2);
    changeMessage(theMessage);
    }
  

  }
