/* Sun Cluster Data Services Builder template version 1.0 */
/*
 * cups_monitor_stop.c - Monitor Stop method for cups
 */

#include <rgm/libdsdev.h>
#include <scha.h>

/*
 * This method stops the fault monitor for a cups resource.
 * This is done via PMF. The PMF tag for the fault monitor is
 * constructed based on RG-name_RS-name.mon.
 */

int
main(int argc, char *argv[])
{

scds_handle_t   scds_handle;
scha_err_t err_code;
scds_pmf_status_t pmf_status;
int instance=0;
int monitor_stop_timeout;

/* Process the arguments passed by RGM and initalize syslog */
if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR) {
        scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
        return (1);
}

/* Check the monitor to see if its running. If not exit 0 */
err_code = scds_pmf_get_status(scds_handle,SCDS_PMF_TYPE_MON,instance, &pmf_status);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Failed to get Monitor Status! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}
if(pmf_status != SCDS_PMF_MONITORED ){
        scds_syslog(LOG_NOTICE,"Service is not being monitored.");
        scds_close(&scds_handle);
        return(0);
}
/* Get the monitor stop timeout. */

monitor_stop_timeout = scds_get_rs_monitor_stop_timeout(scds_handle);

/* The monitor is running, so need to stop the monitor. */
err_code = scds_pmf_stop(scds_handle,SCDS_PMF_TYPE_MON,instance,SIGTERM, monitor_stop_timeout);
if( err_code != SCHA_ERR_NOERR ){
        scds_close(&scds_handle);
        if( err_code == SCHA_ERR_TIMEOUT ){
            scds_syslog(LOG_ERR,"Timeout occured, Failed to Stop Monitor! Error: %s",scha_strerror(err_code));
        }
        else {
            scds_syslog(LOG_ERR,"Failed to Stop Monitor! Error: %s",scha_strerror(err_code));
        }
        return(err_code);
}

/*Free up all the memory allocated by scds_initialize */
scds_close(&scds_handle);

return (0);
}
