/* Sun Cluster Data Services Builder template version 1.0 */
/*
 * cups_svc_stop.c - Stop method for cups
 */

#include <rgm/libdsdev.h>
#include <scha.h>

#define SVC_SMOOTH_PCT          80.0

#define SVC_HARD_PCT            15.0

/*
 * Stops the cups process using PMF
 */

int
main(int argc, char *argv[])
{

scds_handle_t	scds_handle;
scha_err_t err_code;
scds_pmf_status_t pmf_status;
int stop_timeout;
int smooth_stop_timeout;
int hard_stop_timeout;
int instance=0;

/* Process the arguments passed by RGM and initalize syslog */
if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR) {
	scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
	return (1);
}

stop_timeout = scds_get_rs_stop_timeout(scds_handle);
smooth_stop_timeout=(int)((float) stop_timeout * (SVC_SMOOTH_PCT/100));
hard_stop_timeout=(int)((float) stop_timeout * (SVC_HARD_PCT/100));

err_code = scds_pmf_get_status(scds_handle,SCDS_PMF_TYPE_SVC,instance, &pmf_status);
if( err_code != SCHA_ERR_NOERR ){
        scds_syslog(LOG_ERR,"Failed to get PMF status! Error: %s",scha_strerror(err_code));
        scds_close(&scds_handle);
        return(err_code);
}

if( pmf_status != SCDS_PMF_MONITORED ){
        scds_syslog(LOG_NOTICE, "Cups is not Running");
        scds_close(&scds_handle);
	return(0);
}

err_code = scds_pmf_stop(scds_handle,SCDS_PMF_TYPE_SVC,instance,SIGTERM, smooth_stop_timeout);
if( err_code != SCHA_ERR_NOERR ){
        scds_close(&scds_handle);
        if( err_code == SCHA_ERR_TIMEOUT ){
            scds_syslog(LOG_ERR,"Timeout occured, Failed to Stop Cups! Error: %s",scha_strerror(err_code));
	}
	else {
            scds_syslog(LOG_ERR,"Failed to Stop Cups! Error: %s",scha_strerror(err_code));
	}
        return(err_code);
}


scds_syslog(LOG_NOTICE,"Cups service has stopped.");

/* Free up all the memory allocated by scds_initialize */
scds_close(&scds_handle);

/* Return the result of svc_stop method */
return (0);
}
