/* Sun Cluster Data Services Builder template version 1.0 */
/*
 * cups_update.c - Update method for cups
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <rgm/libdsdev.h>

/*
 * Some of the resource properties might have been updated. All such
 * updatable properties are related to the fault monitor. Hence, just
 * restarting the monitor should be enough.
 */

int
main(int argc, char *argv[])
{
	scds_handle_t   scds_handle;
	scha_err_t	err_code;
	int		instance=0; /* set fault monitor instance to 0.*/
	scds_pmf_status_t pmf_status;

	/* Process the arguments passed by RGM and initialize syslog */
	if (scds_initialize(&scds_handle, argc, argv) != SCHA_ERR_NOERR) {
		scds_syslog(LOG_ERR, "Data Service handle failed to initialize!");
		return (1);

	}
	err_code = scds_pmf_get_status(scds_handle,SCDS_PMF_TYPE_MON,instance, &pmf_status);
	if( err_code != SCHA_ERR_NOERR ){
        	scds_syslog(LOG_ERR,"Failed to get Monitor Status! Error: %s",scha_strerror(err_code));
        	scds_close(&scds_handle);
        	return(err_code);
	}
	if(pmf_status != SCDS_PMF_MONITORED ){
        	scds_syslog(LOG_ERR,"Service is not monitored.");
        	scds_close(&scds_handle);
        	return(SCDS_PMF_NOT_MONITORED);
	}
	/*
	 * The Fault monitor is already running and if so stop and
	 * restart it. The second parameter to scds_pmf_restart_fm() uniquely
	 * identifies the instance of the fault monitor that needs to be
	 * restarted.
	 */

	err_code = scds_pmf_restart_fm(scds_handle, instance);
	if (err_code != SCHA_ERR_NOERR) {
		scds_syslog(LOG_ERR,"Failed to restart fault monitor! Error: %s",scha_strerror(err_code));

		/* Free up all the memory allocated by scds_initialize */
		scds_close(&scds_handle);
		return(err_code);
	}

	scds_syslog(LOG_INFO,
	    "Completed successfully.");

	/* Free up all the memory allocated by scds_initialize */
	scds_close(&scds_handle);

	return (0);
}
